package org.bitbucket.bradleysmithllc.etlagent.informatica.dto;

import org.bitbucket.bradleysmithllc.etlagent.informatica.dto.json.informatica.execute.Parameter;
import org.bitbucket.bradleysmithllc.webserviceshubclient.parameter.ParameterFile;
import org.bitbucket.bradleysmithllc.webserviceshubclient.parameter.ScopeSection;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class ExecuteWorkflowRequestDTO extends BaseIntegrationServiceRequestDTO
{
	private final String folder;
	private final String task;
	private final String runInstanceName;
	private String sessionId;

	private final String workflowName;
	private final List<Parameter> parameterList = new ArrayList<Parameter>();

	private final Map<RemoteFile.fileType, List<RemoteFile>> requestFiles = new HashMap<RemoteFile.fileType, List<RemoteFile>>();

	public ExecuteWorkflowRequestDTO(String folder, String workflowName) {
		this(folder, workflowName, null, null, null);
	}

	public ExecuteWorkflowRequestDTO(String folder, String workflowName, ParameterFile parameterFile) {
		this(folder, workflowName, null, null, parameterFile);
	}

	public ExecuteWorkflowRequestDTO(String folder, String workflowName, String task) {
		this(folder, workflowName, task, null, null);
	}

	public ExecuteWorkflowRequestDTO(String folder, String workflowName, String task, ParameterFile parameterFile) {
		this(folder, workflowName, task, null, parameterFile);
	}

	public ExecuteWorkflowRequestDTO(String folder, String workflowName, String task, String runInstance) {
		this(folder, workflowName, task, runInstance, null);
	}

	public ExecuteWorkflowRequestDTO(String folder, String workflowName, String task, String runInstance, ParameterFile parameterFile) {
		this.folder = folder;
		this.workflowName = workflowName;
		runInstanceName = runInstance;
		this.task = task;

		if (parameterFile != null)
		{
			for (ScopeSection section : parameterFile.getSections())
			{
				for (org.bitbucket.bradleysmithllc.webserviceshubclient.parameter.Parameter parameter : section.getParameters())
				{
					Parameter p = new Parameter();

					p.setScope(section.getHeader());
					p.setParameterName(parameter.getParameterName());
					p.setParameterValue(parameter.getParameterValue());

					parameterList.add(p);
				}
			}
		}
	}

	public void addRequestFile(RemoteFile file)
	{
		for (RemoteFile remoteFile : getOrCreate(file.getType()))
		{
			if (remoteFile.sameId(file))
			{
				throw new IllegalArgumentException("Remote file name and fileType already added");
			}
		}

		requestFiles.get(file.getType()).add(file);
	}

	private List<RemoteFile> getOrCreate(RemoteFile.fileType type) {
		List<RemoteFile> list = requestFiles.get(type);

		if (list == null)
		{
			list = new ArrayList<RemoteFile>();

			requestFiles.put(type, list);
		}

		return list;
	}

	public String getTask() {
		return task;
	}

	public String getRunInstanceName() {
		return runInstanceName;
	}

	public String getFolder() {
		return folder;

	}

	public Map<RemoteFile.fileType, List<RemoteFile>> getRequestFiles() {
		return requestFiles;
	}

	public ParameterFile getParameterFile() {
		ParameterFile parameterFile = new ParameterFile();

		for (Parameter parameter : parameterList)
		{
			ScopeSection section = parameterFile.getSection(parameter.getScope());

			if (section == null)
			{
				section = parameterFile.appendSection(parameter.getScope());
			}

			section.appendParameter(parameter.getParameterName(), parameter.getParameterValue());
		}

		return parameterFile;
	}

	public String getWorkflowName() {
		return workflowName;
	}

	public String getSessionId() {
		return sessionId;
	}
}