package org.bitbucket.bradleysmithllc.etlagent.informatica.handler;

import com.fasterxml.jackson.databind.JsonNode;
import org.apache.commons.io.FileUtils;
import org.bitbucket.bradleysmithllc.etlagent.informatica.dto.ExportWorkflowResponseDTO;
import org.bitbucket.bradleysmithllc.etlagent.informatica.dto.json.informatica.export_workflow_to_xml.ExportWorkflowToXmlRequest;
import org.bitbucket.bradleysmithllc.etlagent.informatica.provider.InformaticaDomain;
import org.bitbucket.bradleysmithllc.etlagent.informatica.provider.InformaticaRepository;
import org.bitbucket.bradleysmithllc.etlagent.informatica.provider.InformaticaRepositoryClient;
import org.bitbucket.bradleysmithllc.etlunit.RuntimeSupport;

import java.io.File;

public class ExportWorkflowToXml extends AbstractHandler<ExportWorkflowToXmlRequest, ExportWorkflowResponseDTO> {
	@Override
	public String getId() {
		return "exportWorkflowToXml";
	}

	@Override
	public ExportWorkflowToXmlRequest getRequestContainerObject() {
		return new ExportWorkflowToXmlRequest();
	}

	@Override
	public ExportWorkflowResponseDTO process(JsonNode request, final ExportWorkflowToXmlRequest container) throws Exception {
		System.out.println("Exporting workflow");

		String domain = container.getInformaticaDomain();

		String repo = container.getInformaticaRepository();

		// resolve to an informatica domain
		InformaticaDomain informaticaDomain = informaticaConfiguration.getDefaultDomain();

		if (domain != null) {
			System.out.println("No domain specified - using the default domain");

			informaticaDomain = informaticaConfiguration.getDomain(domain);

			if (informaticaDomain == null) {
				throw new IllegalArgumentException("Domain '" + domain + "' not found");
			}
		} else {
			System.out.println("No domain specified - using the default domain " + informaticaDomain.getDomainName());
		}

		InformaticaRepository informaticaRepository = null;

		if (repo == null) {
			informaticaRepository = informaticaDomain.getDefaultRepository();
		} else {
			informaticaRepository = informaticaDomain.getRepository(repo);
		}

		if (informaticaRepository == null) {
			throw new IllegalArgumentException("Domain '" + domain + "' does not contain repository '" + repo + "'");
		}

		final File temp = runtimeSupport.createTempFile(container.getWorkflowName());

		useRepository(informaticaRepository, new RepositoryVisitor() {
			@Override
			public void useClient(InformaticaRepositoryClient client, RuntimeSupport runtimeSupport) throws Exception {
				client.exportWorkflowToXml(
						container.getWorkflowName(),
						container.getFolder(),
						temp
				);
			}
		});

		ExportWorkflowResponseDTO dto = new ExportWorkflowResponseDTO(FileUtils.readFileToString(temp));

		return dto;
	}
}